==> zmailer-2.99.25-960120.tar.gz: zmailer-2.99.25/scheduler/manual-expirer.c <==
/* manual-expirer -- utility for the ZMailer by Matti Aarnio */

#include <stdio.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include "hostenv.h"
#include "mail.h"
#include <alloca.h>

/*
   manual-expirer  "channel" [ "host" ]

   From the STDIN it eats lines containing names of the files in
   POSTOFFICE/transport/, and then processes each of them by
   looking up "r "/"r~" -entries with matching channel and
   possible host strings (strcmp() processed).
   If the line matches, it is changed to "r+" -- that is, completed
   ok..

   (written for a need at vger.rutgers.edu..  Another quick hack..)

*/

extern int errno;

int process(fd,channel,host)
int fd;
char *host, *channel;
{
	struct stat stbuf;
	char *filebuf;
	char *s, *p, *q, *eof;

	if (fstat(fd,&stbuf) != 0) return 1;

	lseek(fd,0,0);
	filebuf = malloc(stbuf.st_size+2);
	if (!filebuf) return 2; /* HOW COME??? */

	if (read(fd,filebuf,stbuf.st_size) != stbuf.st_size)
	  { free(filebuf); return 3; }
	eof = filebuf + stbuf.st_size;
	s = p = filebuf;
	*eof = 0;

	while (s < eof) {
	  if (*s == 'r') {
	    char *ch, *ho;
	    p = s++;
	    if (*s == ' ' || *s == '~') {
	      /* We MAY have some job! */
	      ++s;
	      while (*s == ' ' || (*s >= '0' && *s <= '9')) ++s;
	      /* Now 's' points at the start of the channel name */
	      ch = s;
	      while (s < eof && *s != '\n' && *s != ' ' && *s != '\t') ++s;
	      if (*s != '\n')
		*s++ = 0;
	      if (host != NULL) {
		while (s < eof && (*s == ' ' || *s == '\t')) ++s;
		ho = s;
		while (s < eof && *s != '\n' && *s != ' ' && *s != '\t') ++s;
		if (*s != '\n')
		  *s = 0;
	      } else
		ho = NULL;
	      while (s < eof && *s != '\n') ++s;

	      /* Ok, "ch" and "ho" are found.. */
	      if (strcmp(ch, channel) == 0) {
		int doit = 0;
		if (host != NULL &&
		    strcmp(ho, host) == 0) {
		  doit = 1;
		} else if (host == NULL)
		  doit = 1;
		if (doit) {
		  lseek(fd,(p - filebuf)+1,0);
		  if (write(fd,"+",1) != 1) {
			free(filebuf);
			return 9;
		  }
		}
	      }
	    }
	  } else if (*s == 'm') {
	    /* rewritten headers, scan until "\n\n" is found! */
	    while (s < eof && s[0] == '\n' && s[1] == '\n') ++s;
	    ++s; /* To the second '\n' */
	  } else {
	    /* Any other (one line) input files.. */
	    while (s < eof && *s != '\n') ++s;
	  }
	  ++s; /* Skip the trailing '\n' */
	}
	free(filebuf);
	return 0;
}

int main(argc,argv)
int argc;
char *argv[];
{
	int fd, rc;
	char filename[512];
	char *channel = argv[1];
	char *host    = NULL;
	char *s;
	int err;

	if (argc < 2 || argc > 3) {
	  fprintf(stderr,"manual-expirer channelname [hostname] -- stdin eats filenames\n");
	  return 64;
	}
	if (argc == 3)
	  host = argv[2];

	while (!feof(stdin)) {
	  if (fgets(filename,sizeof(filename),stdin) == NULL)
	    break;
	  s = strchr(filename,'\n');
	  if (s) *s = 0;
	  else break; /* Not newline-terminated input! */
	  fd = open(filename,O_RDWR,0);
	  if (fd < 0) {
	    err = errno;
	    fprintf(stderr,"File '%s' could not be opened",filename);
	    errno = err;
	    perror(":");
	    continue;
	  }
	  rc = process(fd, channel, host);
	  if (rc != 0) {
	    err = errno;
	    fprintf(stderr,"File '%s' processing returned code %d",
		    filename,rc);
	    errno = err;
	    perror("error code");
	  }
	  close(fd);
	}
	return 0;
}

==> zmailer-2.99.48.tar.gz: zmailer-2.99.48/scheduler/manual-expirer.c <==
/* manual-expirer -- utility for the ZMailer by Matti Aarnio */

#include <stdio.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include "hostenv.h"
#include "mail.h"
#include <alloca.h>

/*
   manual-expirer  "channel" [ "host" ]

   From the STDIN it eats lines containing names of the files in
   POSTOFFICE/transport/, and then processes each of them by
   looking up "r "/"r~" -entries with matching channel and
   possible host strings (strcmp() processed).
   If the line matches, it is changed to "r+" -- that is, completed
   ok..

   (written for a need at vger.rutgers.edu..  Another quick hack..)

*/

extern int errno;

int process(fd,channel,host)
int fd;
char *host, *channel;
{
	struct stat stbuf;
	char *filebuf;
	char *s, *p, *q, *eof;

	if (fstat(fd,&stbuf) != 0) return 1;

	lseek(fd,(off_t)0,0);
	filebuf = malloc(stbuf.st_size+2);
	if (!filebuf) return 2; /* HOW COME??? */

	if (read(fd,filebuf,stbuf.st_size) != stbuf.st_size)
	  { free(filebuf); return 3; }
	eof = filebuf + stbuf.st_size;
	s = p = filebuf;
	*eof = 0;

	while (s < eof) {
	  if (*s == 'r') {
	    char *ch, *ho;
	    p = s++;
	    if (*s == ' ' || *s == '~') {
	      /* We MAY have some job! */
	      ++s;
	      while (*s == ' ' || (*s >= '0' && *s <= '9')) ++s;
	      /* Now 's' points at the start of the channel name */
	      ch = s;
	      while (s < eof && *s != '\n' && *s != ' ' && *s != '\t') ++s;
	      if (*s != '\n')
		*s++ = 0;
	      if (host != NULL) {
		while (s < eof && (*s == ' ' || *s == '\t')) ++s;
		ho = s;
		while (s < eof && *s != '\n' && *s != ' ' && *s != '\t') ++s;
		if (*s != '\n')
		  *s = 0;
	      } else
		ho = NULL;
	      while (s < eof && *s != '\n') ++s;

	      /* Ok, "ch" and "ho" are found.. */
	      if (strcmp(ch, channel) == 0) {
		int doit = 0;
		if (host != NULL &&
		    strcmp(ho, host) == 0) {
		  doit = 1;
		} else if (host == NULL)
		  doit = 1;
		if (doit) {
		  lseek(fd,(off_t)(p - filebuf)+1,0);
		  if (write(fd,"+",1) != 1) {
			free(filebuf);
			return 9;
		  }
		}
	      }
	    }
	  } else if (*s == 'm') {
	    /* rewritten headers, scan until "\n\n" is found! */
	    while (s < eof && s[0] == '\n' && s[1] == '\n') ++s;
	    ++s; /* To the second '\n' */
	  } else {
	    /* Any other (one line) input files.. */
	    while (s < eof && *s != '\n') ++s;
	  }
	  ++s; /* Skip the trailing '\n' */
	}
	free(filebuf);
	return 0;
}

int main(argc,argv)
int argc;
char *argv[];
{
	int fd, rc;
	char filename[512];
	char *channel = argv[1];
	char *host    = NULL;
	char *s;
	int err;

	if (argc < 2 || argc > 3) {
	  fprintf(stderr,"manual-expirer channelname [hostname] -- stdin eats filenames\n");
	  return 64;
	}
	if (argc == 3)
	  host = argv[2];

	while (!feof(stdin)) {
	  if (fgets(filename,sizeof(filename),stdin) == NULL)
	    break;
	  s = strchr(filename,'\n');
	  if (s) *s = 0;
	  else break; /* Not newline-terminated input! */
	  fd = open(filename,O_RDWR,0);
	  if (fd < 0) {
	    err = errno;
	    fprintf(stderr,"File '%s' could not be opened",filename);
	    errno = err;
	    perror(":");
	    continue;
	  }
	  rc = process(fd, channel, host);
	  if (rc != 0) {
	    err = errno;
	    fprintf(stderr,"File '%s' processing returned code %d",
		    filename,rc);
	    errno = err;
	    perror("error code");
	  }
	  close(fd);
	}
	return 0;
}
